window.addEventListener("load", function(event) {
	window.removeEventListener("load", arguments.callee, false);

	// if titlebar does not exist, just exit.
	var titlebar = document.getElementById("titlebar");
	if (!titlebar) return;
	var titlebox = document.getElementById("titlebar-content");
	if (!titlebox) return;
	var fxbtnbox = document.getElementById("appmenu-button-container");
	if (!fxbtnbox) return;

	// get the required elements
	var appmenu = document.getElementById("appmenu-popup");
	var toolbox = document.getElementById("navigator-toolbox");
	var menubar = document.getElementById("toolbar-menubar");
	var menubox = document.getElementById("menubar-items");
	var tabsbar = document.getElementById("TabsToolbar");
	var tabsbox = document.getElementById("tabbrowser-tabs");
	var ctxmenu = document.getElementById("toolbar-context-menu");
	var cmdcumz = document.getElementById("cmd_CustomizeToolbars");

	// create the personal-titlebar container on titlebox
	var container = document.createElement("toolbar");
	container.id = "personal-titlebar";
	container.setAttribute("iconsize", "small");
	container.setAttribute("defaulticonsize", "small");
	container.setAttribute("lockiconsize", "true");
	titlebox.insertBefore(container, fxbtnbox.nextSibling);

	// apply toolbar's context-menu to titlebox
	titlebox.setAttribute("context", "toolbar-context-menu");

	// force to disable the "TabsInTitlebar" feature
	TabsInTitlebar.allowedBy("personal-titlebar", false);

	// function: update the titlebar item's text-color
	function updateTitlebarItemsTextColor() {
		container.setAttribute("textcolor", Services.prefs.getBoolPref("extensions.personaltitlebar.textcolor"));
	}

	// function: update the NavToolbox's position
	function updateNavToolboxPosition() {
		if (document.documentElement.getAttribute("sizemode") == "normal") {
			titlebar.style.marginBottom = - (titlebar.boxObject.height - titlebox.boxObject.height) + "px";
		} else {
			titlebar.style.marginBottom = "";
		}
	}

	// function: dispatch "aftercustomization" event
	function dispatchAfterCustomizationEvent() {
		var evt = document.createEvent("Events");
		evt.initEvent("aftercustomization", true, true);
		toolbox.dispatchEvent(evt);
	}

	// function: move menubar items to titlebox or restore to menubar
	function updateMenubarItemsPosition(forceRestore) {
		// before move any menubar items, uninit the PlacesToolbar
		PlacesToolbarHelper.customizeStart();
		BookmarksMenuButton.customizeStart();
		// decide to move or restore
		var isMoveToTitleBar = !forceRestore && !titlebar.hidden;
		if (isMoveToTitleBar) {
			// move to titlebar
			container.setAttribute("mode", menubar.getAttribute("mode"));
			container.setAttribute("currentset", menubar.getAttribute("currentset"));
			while (menubar && menubar.firstChild) {
				container.appendChild(menubar.firstChild);
			}
			menubar.setAttribute("customizable", false);
			menubar.setAttribute("currentset", "__empty");
			menubar.hidden = true;
		} else {
			// restore to menubar
			while (container && container.firstChild) {
				menubar.appendChild(container.firstChild);
			}
			menubar.setAttribute("customizable", true);
			menubar.setAttribute("currentset", container.getAttribute("currentset"));
			menubar.hidden = false;
		}
		// after move the menubar items, reinit the PlacesToolbar
		PlacesToolbarHelper.customizeDone();
		BookmarksMenuButton.customizeDone();
		// prevent user remove the menubox or tabsbox
		menubox.removeAttribute("removable");
		tabsbox.removeAttribute("removable");
		// prevent tabsbar collapsed
		tabsbar.removeAttribute("collapsed");
		// update the NavToolbox's position
		updateNavToolboxPosition();
		// dispatch "aftercustomization" event
		dispatchAfterCustomizationEvent();
	}

	// function: update the titletext's value
	function updateTitleTextValue(customizing) {
		setTimeout(function() {
			var titletext = document.getElementById("titlebar-text");
			if (titletext) titletext.value = customizing ? titletext.parentNode.getAttribute("title") : document.documentElement.getAttribute("title");
		}, 0);
	}

	// function: update the titletext's align
	function updateTitleTextAlign() {
		var titletext = document.getElementById("titlebar-text");
		if (titletext) titletext.style.textAlign = (function() {
			switch (Services.prefs.getIntPref("extensions.personaltitlebar.titletext.align")) {
				case 1:
					return "center";
				case 2:
					return "end";
				default:
					return "start";
			}
		})();
	}

	// function: add an observer for the preference modified event
	function addPrefModifiedObserver(prefName, callback) {
		const Cc = Components.classes, Ci = Components.interfaces;
		const prefService = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch2).QueryInterface(Ci.nsIPrefService);
		prefService.addObserver(prefName, { observe: callback }, false);
	}

	// observe the preference for changing titletext's align
	addPrefModifiedObserver("extensions.personaltitlebar.titletext.align", updateTitleTextAlign);

	// observe the preference for changing titletext's align
	addPrefModifiedObserver("extensions.personaltitlebar.textcolor", updateTitlebarItemsTextColor);

	// function: add an observer for the attribute modified event
	function addAttrModifiedObserver(elmt, attrName, callback) {
		elmt.addEventListener("DOMAttrModified", function(event) {
			if (event.target != elmt) return;
			if (event.attrName != attrName) return;
			callback(event);
		}, true);
	}

	// observe cmd_CustomizeToolbars's "disabled" attribute for update the menubar items position
	addAttrModifiedObserver(cmdcumz, "disabled", function(event) {
		var customizing = (event.newValue == "true");

		updateMenubarItemsPosition(customizing); // if is customizing, force restore the menubar items
		updateTitleTextValue(customizing); // if is customizing, update the titletext's label
		ctxmenu.hidden = customizing; // if is customizing, disable the context menu
	});

	// observe titlebar's "hidden" attribute for update the menubar items position
	addAttrModifiedObserver(titlebar, "hidden", function(event) { updateMenubarItemsPosition(); });

	// observe document title's change to update titletext toolbaritem
	addAttrModifiedObserver(document.documentElement, "title", function(event) { updateTitleTextValue(); });

	// on startup, update the titlebar item's text-color
	updateTitlebarItemsTextColor();

	// on startup, update the menubar items position
	updateMenubarItemsPosition();

	// on startup, update the titletext's align
	updateTitleTextAlign();

	// on startup, update the titletext's label
	updateTitleTextValue();

	// function: toggle the menubar menus display
	function toggleMainMenusVisibility() {
		if (menubox.getAttribute("invisible") != "true") {
			menubox.setAttribute("invisible", "true");
		} else {
			menubox.removeAttribute("invisible");
		}
		document.persist(menubox.id, "invisible");
		// if PlaceToolbar existed, update the PlaceToolbar's Chevron
		if (PlacesToolbarHelper._viewElt && PlacesToolbarHelper._viewElt._placesView) {
			PlacesToolbarHelper._viewElt._placesView._updateChevronTimerCallback();
		}
	}

	// implement: press [F10] toggle menubar menus display
	window.addEventListener("keyup", function(event) {
		var keyUpF10 = (event.keyCode == KeyEvent.DOM_VK_F10);
		if (keyUpF10) toggleMainMenusVisibility();
	}, true);

	// on resize, update the NavToolbox's position
	window.addEventListener("resize", function(event) {
		if (event.target != window) return;
		updateNavToolboxPosition();
	}, false);

}, false);
